"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("./types");
const builder_1 = require("./builder");
function createBuilderClass() {
    return (template) => {
        const DynamicBuilder = class extends builder_1.BuilderBase {
            constructor() {
                super(template);
            }
        };
        function defineScalarSetter(fieldName) {
            DynamicBuilder.prototype[fieldName] = function (value) {
                return this.setScalar(fieldName, value);
            };
        }
        function defineNestedSetter(fieldName, builderClass) {
            DynamicBuilder.prototype[fieldName] = function (block) {
                const builder = new builderClass();
                return this.setNested(fieldName, builder, block);
            };
        }
        for (const fieldName in template) {
            const fieldTemplate = template[fieldName];
            if (types_1.isNested(fieldTemplate)) {
                defineNestedSetter(fieldName, fieldTemplate.nested);
            }
            else {
                defineScalarSetter(fieldName);
            }
        }
        return DynamicBuilder;
    };
}
exports.createBuilderClass = createBuilderClass;
//# sourceMappingURL=create.js.map